/***************************************************************************
 *   Copyright (C) 2009 by Rico Rommel   *
 *   rico@bierrommel.de   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "btdeviceinterface.h"
#include <QDebug>
#include <QDBusReply>
#include <QStringList>

BTDeviceInterface::BTDeviceInterface(QString name, QWidget *parent, QStringList states)
 : DeviceInterface(name, parent, states)
{
	
	getManagerInterface();
	

}


BTDeviceInterface::~BTDeviceInterface()
{
}




QString BTDeviceInterface::getDefaultAdapter()
{
	QString path;
	QDBusObjectPath objectpath;

	if ( ManagerInterface->isValid () ) {
		QDBusReply<QDBusObjectPath> reply;
		reply = ManagerInterface->call("DefaultAdapter");

		objectpath = reply;	
		path = objectpath.path();
//		qDebug() << "Bluetooth Adapter Path is" << path;
		}

	else 	{
//		qDebug() << "no Bluetooth Adapter found or bluetoothd not running";
		}

	return path;

}

QString BTDeviceInterface::getPreventState()
{
	QString state;
	state = "none";

	if (CheckButton->isChecked() ) {

		if (!ManagerInterface->isValid()) getManagerInterface();

		
		if (ManagerInterface->isValid()) {
			QString adapterpath = getDefaultAdapter();
			if (!adapterpath.isEmpty()) {
				QStringList DeviceList = getDeviceList(adapterpath);

				foreach(const QString &devicepath, DeviceList) {
					if (getConnectionStatus(devicepath)) state = ComboBox->currentText ();
				}
			}
//			else qDebug() << "Bluetooth: Could not get default adapter";
				
		}

	}

	
	return state;
}


void BTDeviceInterface::getManagerInterface()
{
    	ManagerInterface = new QDBusInterface("org.bluez", "/", "org.bluez.Manager", QDBusConnection::systemBus(), this);

}


QStringList BTDeviceInterface::getDeviceList(QString adapterpath)
{

	QStringList DeviceList;

	QDBusInterface *Adapterinterface = new QDBusInterface("org.bluez", adapterpath, "org.bluez.Adapter", QDBusConnection::systemBus(), this);

	if (Adapterinterface->isValid()) {	
		QDBusReply< QList < QDBusObjectPath > > reply;
		reply = Adapterinterface->call("ListDevices");
	
		foreach(const QDBusObjectPath &device,reply.value())
			DeviceList.append(device.path());
	}

	return DeviceList;
	
}


bool BTDeviceInterface::getConnectionStatus(QString devpath)
{
    	QDBusInterface *devinterface;
	devinterface = new QDBusInterface("org.bluez", devpath, "org.bluez.Device",QDBusConnection::systemBus(), this);

	QDBusReply< QMap < QString,QVariant> > reply;
	bool conn;
	reply = devinterface->call("GetProperties");
	conn = reply.value().value("Connected").toBool();

//	qDebug() << reply.value().value("Alias").toString() << " " << conn;
	
	return conn;

}
